/* SPI Slave example, sender (uses SPI master driver)

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/
#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include "esp_log.h"
#include "driver/spi_master.h"
#include "driver/gpio.h"
#include "esp_timer.h"
#include <sys/unistd.h>


#define GPIO_MOSI           23
#define GPIO_MISO           19
#define GPIO_SCLK           18
#define GPIO_CS             5

#define ESP_HOST VSPI_HOST
esp_err_t ret;
spi_device_handle_t spi_handle;

void SPI_Init (void)
{
    //Configuration for the SPI bus
    spi_bus_config_t buscfg = {
        .mosi_io_num = GPIO_MOSI,
        .miso_io_num = GPIO_MISO,
        .sclk_io_num = GPIO_SCLK,
        .quadwp_io_num = -1,
        .quadhd_io_num = -1
    };

    //Configuration for the SPI device on the other side of the bus
    spi_device_interface_config_t devcfg = {
        .command_bits = 0,
        .address_bits = 0,
        .dummy_bits = 0,
        .clock_speed_hz = 2000000,
        .duty_cycle_pos = 128,      //50% duty cycle
        .mode = 0,
        .spics_io_num = GPIO_CS,
        .queue_size = 3
    };
    
    //Initialize the SPI bus and add the device we want to send stuff to.
    ret = spi_bus_initialize(ESP_HOST, &buscfg, SPI_DMA_CH_AUTO);
    assert(ret == ESP_OK);
    ret = spi_bus_add_device(ESP_HOST, &devcfg, &spi_handle);
    assert(ret == ESP_OK);    
}

void SPI_Transmit (uint8_t *data, int bytes)
{
    spi_transaction_t trans;
    memset(&trans, 0, sizeof(trans));
    
    trans.tx_buffer = data;
    trans.length = bytes*8;
    
    if (spi_device_transmit(spi_handle, &trans) != ESP_OK)
    {
		printf ("writing ERROR\n");
	}	
}

void SPI_Receive (uint16_t cmd, size_t cmdLen, uint64_t Addr, size_t addrLen, uint8_t *buffertoStore, size_t dataLen)
{
    spi_transaction_t trans;
    memset(&trans, 0, sizeof(trans));
    
    spi_transaction_ext_t trans_ext;
    memset(&trans_ext, 0, sizeof(trans_ext));
    
    trans.flags = SPI_TRANS_VARIABLE_ADDR | SPI_TRANS_VARIABLE_CMD;
    trans.cmd = cmd;
    trans.addr = Addr;
    trans.rx_buffer = buffertoStore;
    trans.length = 	(dataLen)*8;
    
    trans_ext.address_bits = addrLen*8;
    trans_ext.command_bits = cmdLen*8;
    trans_ext.base = trans;
    
    if (spi_device_transmit(spi_handle, (spi_transaction_t *)&trans_ext) != ESP_OK)
    {
		printf ("writing ERROR\n");
	}	
}

void write_enable (void)
{
	uint8_t cmd = 0x06;
	SPI_Transmit(&cmd, 1);
	usleep(5000);  // wait 5ms
}

void sector_erase (int numsector)
{
	uint8_t cmd = 0x20;
	uint32_t memAddress = numsector*16*256;  // sector num x 16 pages x 256 bytes
	uint8_t tData[10];
	tData[0] = cmd;
	tData[1] = (memAddress>>16)&0xFF;
	tData[2] = (memAddress>>8)&0xFF;
	tData[3] = (memAddress)&0xFF;
	
	write_enable();
	SPI_Transmit(tData, 4);
	sleep(1);
}

void write_data (int pagenum, uint8_t *datatoWrite, size_t bytes)
{
	uint8_t cmd = 0x02;
	uint32_t memAddress = pagenum*256; 
	uint8_t tData[bytes+4];  // 4 additional bytes-> 1cmd, 3addr
	tData[0] = cmd;
	tData[1] = (memAddress>>16)&0xFF;
	tData[2] = (memAddress>>8)&0xFF;
	tData[3] = (memAddress)&0xFF;
	
	for (int i=0; i<bytes; i++)
	{
		tData[i+4] = datatoWrite[i];
	}
	
	write_enable();
	SPI_Transmit(tData, bytes+4);
	usleep(200000);
}

void read_data (int pagenum, uint8_t *buffertoStore, size_t bytes)
{
	uint8_t cmd = 0x03;
	uint32_t memAddress = pagenum*256; 
	SPI_Receive(cmd, 1, memAddress, 3,  buffertoStore, bytes);
}
	
char *data = "Hello world 12345";
uint8_t RxData[100];


void app_main(void)
{
	SPI_Init();
	sector_erase(0);
	write_data(0, (uint8_t *)data, 17);
	read_data(0, RxData, 17);
	printf ("received = %s\n", RxData);
	
	while (1)
	{
		sleep(2);
	}
}
